#![forbid(unsafe_code)]

pub mod libdit {
	use std::process::exit;
	use std::str::FromStr;

	/// Represents a time in a StarDIT day
	#[derive(Debug, PartialEq)]
	pub struct Dit {
		pub deca: u16,
		pub decim: u16,
		pub desec: u16,
	}

	impl FromStr for Dit {
		type Err = std::num::ParseIntError;

		fn from_str(dit: &str) -> Result<Self, Self::Err> {
			if dit.len() != 7 {
				eprintln!("Wrong DIT length");
				exit(1);
			}

			Ok(Dit {
				deca: match dit[0..1].parse() {
					Ok(deca) => deca,
					Err(_) => {
						eprintln!("Wrong deca");
						exit(1);
					}
				},
				decim: match dit[2..4].parse() {
					Ok(decim) => decim,
					Err(_) => {
						eprintln!("Wrong decim");
						exit(1);
					}
				},
				desec: match dit[5..7].parse() {
					Ok(desec) => desec,
					Err(_) => {
						eprintln!("Wrong desec");
						exit(1);
					}
				},
			})
		}
	}

	/// Returns relative progress in the day, between 0 and 1 corresponding to `unix`
	pub fn unix_to_dit(unix: f64) -> f64 {
		(unix - (12.0 * 3600.0)) % 86400.0 / 86400.0
	}

	/// Returns relative progress in the day, between 0 and 1 corresponding to `dit`
	pub fn dit_struct_to_dit(dit: Dit) -> f64 {
		((dit.deca as f32 / 10.0) + (dit.decim as f32 / 1000.0) + (dit.desec as f32 / 100000.0))
			.into()
	}

	/// Returns a struct containing deca, decim and desec corresponding to `unix`
	pub fn unix_to_dit_struct(unix: f64) -> Dit {
		let dit = unix_to_dit(unix);
		Dit {
			deca: (dit * 10.0 % 10.0) as u16,
			decim: (dit * 1000.0 % 100.0) as u16,
			desec: (dit * 100000.0 % 100.0) as u16,
		}
	}

	/// Returns a unix timestamp corresponding to `dit`
	pub fn dit_struct_to_unix(dit: Dit) -> f64 {
		dit_struct_to_dit(dit) * 86400.0 + (12.0 * 3600.0)
	}

	/// Returns a string containing "deca.decim.desec" corresponding to `unix`
	///
	/// # Examples
	///
	/// ```
	/// use libdit::libdit::unix_to_dit_string;
	/// assert_eq!("9.99.99", unix_to_dit_string(1672574399.9999));
	/// ```
	pub fn unix_to_dit_string(unix: f64) -> String {
		let dit = unix_to_dit_struct(unix);
		format!("{}.{:0>2}.{:0>2}", dit.deca, dit.decim, dit.desec,)
	}
}
